#define HEIGHT 1080
#define WIDTH 1920
#define FILEINAME "route85_1920x1080.yuv"
#define ONAME "OptFlow"

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include "sds_lib.h"
#include "optflow.h"

pix_t fileBuf[WIDTH];

void read_yuv_frame(pix_t *buf, int s, int w, int h, FILE *f);
void write_yuv_file(yuv_t *buf, int s, int w, int h, char *name);
void convert_Y8toCY16(pix_t *Ybuf, yuv_t *CYbuf, int pixct);
void convert_CY16toY8(yuv_t *CYbuf, pix_t *Ybuf, int pixct);

// monochrome yuv422 file is read using width w * height h
// only Y data is read, and returned in unsigned char buffer
// stride s is >= width w
// use buffer pix_t fileBuf[WIDTH];
// skip over U and V planes

void read_yuv_frame(pix_t *buf, int s, int w, int h, FILE *f)
{
    int i,j;
    
    // store Y
    for (j=0;j<h;j++) {
        fread(fileBuf, w, 1, f);
        for (i=0;i<w;i++) {
            *(buf+j*s+i) = fileBuf[i];
        }
    }

    // skip over U
    for (j=0;j<h;j++)
        fread(fileBuf, w/2, 1, f);

    // skip over V
    for (j=0;j<h;j++)
        fread(fileBuf, w/2, 1, f);

}

// monochrome yuv422 file is written from Y data using width w * height h
// w * h is appended to name, recognized by YUV Player Deluxe and other players
// stride s is >= width w
// use buffer unsigned char fileBuf[WIDTH];
void write_yuv_file(yuv_t *buf, int s, int w, int h, char *name)
{
    int i,j;
    char fname[32];
    FILE *f;
	yuv_t yuvval;
    
    sprintf(fname,"%s_%dx%d.yuv",name,w,h);
    f = fopen(fname,"wb");
    if (f==NULL) {
        printf("failed to open file %s\n", fname);
        return;
    }
    else {
        printf("wrote file %s\n", fname);
    }

    // Y
    for (j=0;j<h;j++) {
        for (i=0;i<w;i++) {
            yuvval = *(buf+j*s+i);
            fileBuf[i] = (pix_t) (yuvval & 0x00FF);
        }
        fwrite(fileBuf, w, 1, f);
    }

    // U
    for (j=0;j<h;j++) {
        for (i=0;i<w/2;i++) {
            yuvval = *(buf+j*s+(i*2));
            fileBuf[i] = (pix_t) ((yuvval>>8) & 0x00FF);
        }
        fwrite(fileBuf, w/2, 1, f);
    }

    // V
    for (j=0;j<h;j++) {
        for (i=0;i<w/2;i++) {
            yuvval = *(buf+j*s+(i*2+1));
            fileBuf[i] = (pix_t) ((yuvval>>8) & 0x00FF);
        }
        fwrite(fileBuf, w/2, 1, f);
    }

    fclose(f);
}

void convert_Y8toCY16(pix_t *Ybuf, yuv_t *CYbuf, int pixct)
{
	int i;
	yuv_t cypix;

	for (i=0; i<pixct; ++i) {
		cypix = 0x8000 | Ybuf[i];
		CYbuf[i] = cypix;
	}
}

int main()
{
	FILE *f;
	pix_t *inY1 = (pix_t *)sds_alloc_non_cacheable(HEIGHT*WIDTH);
	yuv_t *inCY1 = (yuv_t *)sds_alloc_non_cacheable(HEIGHT*WIDTH*2);
	pix_t *inY2 = (pix_t *)sds_alloc_non_cacheable(HEIGHT*WIDTH);
	yuv_t *inCY2 = (yuv_t *)sds_alloc_non_cacheable(HEIGHT*WIDTH*2);
	yuv_t *outCY = (yuv_t *)sds_alloc_non_cacheable(HEIGHT*WIDTH*2);
	printf("allocated buffers\n");

	f = fopen(FILEINAME,"rb");
	if (f == NULL) {
		printf("failed to open file %s\n", FILEINAME);
		return -1;
	}
	printf("opened file %s\n", FILEINAME);

	read_yuv_frame(inY1, WIDTH, WIDTH, HEIGHT, f);
	printf("read 1st %dx%d frame\n", WIDTH, HEIGHT);
	read_yuv_frame(inY2, WIDTH, WIDTH, HEIGHT, f);
	printf("read 2nd %dx%d frame\n", WIDTH, HEIGHT);

	fclose(f);
	printf("closed file %s\n", FILEINAME);

	convert_Y8toCY16(inY1, inCY1, HEIGHT*WIDTH);
	printf("converted 1st frame to 16bit\n");
	convert_Y8toCY16(inY2, inCY2, HEIGHT*WIDTH);
	printf("converted 2nd frame to 16bit\n");

	fpga_optflow(inCY1, inCY2, outCY, HEIGHT, WIDTH, WIDTH, 10.0);
	printf("computed optical flow\n");

	// write optical flow data image to disk
	write_yuv_file(outCY, WIDTH, WIDTH, HEIGHT, ONAME);

	sds_free(inY1);
	sds_free(inCY1);
	sds_free(inY2);
	sds_free(inCY2);
	sds_free(outCY);
	printf("freed buffers\n");

	printf("TEST PASSED\n");
	return 0;
}
